/**********************************************
**
**
**	Defines for AmiSoft
**
**
***********************************************/

#include	<stdio.h>
#include	"./amiver.h"

#define         SftName         "AmiBIOSDeco"                                   
#define         SftEMail        "Anton Borisov, anton.borisov@gmail.com"

        /********SoftWare*************/                                         
        byte SoftName[]         = "-="SftName", version "SftVersion"=-";
        byte CopyRights[]       = "\n(C) Anton Borisov, 2000, 2002-2003, 2006. Portions (C) 1999-2000";
	byte Url[]              = "Bug-reports direct to "SftEMail;
